
/*
 * $Id: CltOpts.cc,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "CltOpts.h"
#include "UniqId.h"
#include "SharedOpts.h"

#include "globals.h"

CltOpts TheCltOpts;


CltOpts::CltOpts():
	theOrgAddr(this,        "origin <host:port>", "origin host", NetAddr("localhost", 8080)),
	thePxyAddr(this,        "proxy <host:port>",  "proxy host", NetAddr("127.0.0.1", 8080)),

	theWorldType(this,      "world_type <str>",   "part of a URL", "Polygraph"),
	theWorldId(this,        "world_id <str>",     "part of a URL", Id2Str(UniqId::Create())),
	theWorldCap(this,       "world_cap <max|n>",  "#unique object ids", 0x7fffffff),
	theWorldUrlsFileName(this,"world_urls <file>","file with URLs"),

	theUniqStart(this,      "unique_urls <offset>", "append unique suffix (counter) to URLs"),
	theReqOrder(this,       "order <int>",        "request order", 1),

	thePorts(this,          "ports <eph|hi:lo>",  "ephemeral|range"),
	theConnPoolCap(this,    "conn_pool <capacity>","#concurrent connections (max or start)"),
	adjustConnPoolSize(this,"conn_pool_dyn <n|y>","dynamically adjust the size of a conn pool"),
	theConnEstTimeout(0,    "conn_est_tout <tm>", "estab. timeout"),
	theLaunchWindow(this,   "launch_win <time>",  "start all within"),

	theReqCcl(this,         "req_ccl <int>",      "#concurrent reqs"),
	doIOs(this,             "do_ios <yes|no>",    "do socket ios", true)
{
}

bool CltOpts::validate() const {
	if (theReqCcl < 0) {
		cerr << ThePrgName << ": unknown request concurrency level" << endl;
		return false;
	}
	if (TheOpts.theGoal < 0) {
		cerr << ThePrgName << ": undefined goal" << endl;
		return false;
	}
	return true;
}
