
/*
 * $Id: CltXact.cc,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <strstream.h>

#include "Clock.h"
#include "Connection.h"
#include "StatPing.h"
#include "StatPhaseMgr.h"
#include "SharedOpts.h"
#include "World.h"
#include "CltOpts.h"
#include "Client.h"
#include "CltXact.h"
#include "globals.h"


void CltXact::reset() {
	Xaction::reset();
	theOwner = 0;
	theRepSize = theConsumedSize = 0;
	theState = stNone;

	theRepHdr.reset();
}

void CltXact::exec(Client *anOwner, Connection *aConn) {
	Assert(anOwner);
	theOwner = anOwner;

	start(aConn);

	theId.create();
	makeReq(theConn->theWrBuf);
	theState = stSpaceWaiting;
	noteWriteReady(-1); // kludge, sort of 
}

// XXX: should accept errno
void CltXact::finish(bool ok) {
	stop(ok ? 0 : -1); // errno
	
	if (ok)
		TheStatPhaseMgr->xactRdSz(theConsumedSize, theLifeTime);

	theOwner->noteXactDone(this);
}

void CltXact::noteReadReady(int) {
	theConn->read();

	if (theConn->bad()) {
		finish(false);
		return;
	}

	if (theState == stHdrWaiting) {
		if (theRepHdr.parse(theConn->theRdBuf.content(), theConn->theRdBuf.contSize())) {

			// dump reply headers
			if (TheOpts.doPrintReps)
				cout.write(theConn->theRdBuf.content(), theRepHdr.theHdrSize);

			if (theRepHdr.theContSize < 0) { // no content len
				finish(false);
				return;
			}

			theRepSize = theRepHdr.theHdrSize + theRepHdr.theContSize;

			if (!theRepHdr.theXactId) { // foreign reply; read it anyway?
				;
			} else // note: do not count foreign replies as misses or hits!
			if (theId.myMutant(theRepHdr.theXactId)) {
				CallStat(ThePinger, miss(theRepSize));
			} else {
				CallStat(ThePinger, hit(theRepSize));
			}

			theState = stBodyWaiting;
		} else
		if (theConn->theRdBuf.full()) {
			finish(false); // too big header
			return;
		}
	}

	if (theState == stBodyWaiting) {
		theConsumedSize += theConn->theRdBuf.contSize();
		theConn->theRdBuf.consumedAll();
		if (theConsumedSize == theRepSize) {
			finish(theRepHdr.theXactId); // account for foreign replies here
			return;
		}
	}

	if (theConn->exhausted()) { // premature eof
		finish(false);
		return;
	}
}

void CltXact::noteWriteReady(int) {

	Assert(theConn->theWrBuf.contSize() > 0);
	theConn->write();

	if (theConn->bad())
		finish(false);
	else
	if (theConn->theWrBuf.empty()) {
		theConn->theWr.stop(this);
		theState = stHdrWaiting;
		theConn->theRd.start(this);
	} else
	if (!theConn->theWr.theReserv)
		theConn->theWr.start(this);
}

void CltXact::makeReq(WrBuf &buf) {
	static int oid_off = Max(0, TheCltOpts.theUniqStart - 1);
	const int oid = TheWorld.oid(oid_off);

	// XXX: strstream is not the most efficient implementation
	// XXX: can be further optimized with pre-build headers and Strings

	const char *reqStart = buf.space();
	ostrstream os(buf.space(), buf.spaceSize());

	// request line prefix
	const NetAddr &orig = TheCltOpts.theOrgAddr;
	os << "GET http://" << orig.host();
	if (orig.port() != 80)
		os << ':' << orig.port();

	// url
	if (const char *url = TheWorld.url(oid)) {
		os << url; // preloaded
	} else {
		os << "/_" << oid << '/' << TheWorld.id() << '/' << TheWorld.type();

		if (TheCltOpts.theUniqStart > 0)
			os << '/' << TheWorld.uid(oid_off);
	}

	os 
		<< " HTTP/1.0" << crlf
		<< "Accept: */*" << crlf
		<< "X-Xact: " << theId << crlf
		<< crlf;

	oid_off++;
	buf.appended(os.tellp());

	// dump request
	if (TheOpts.doPrintReqs)
		cout.write(reqStart, buf.space()-reqStart);
}
