
/*
 * $Id: ILog.cc,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "LogMeta.h"
#include "ILog.h"
#include "logTags.h"
#include "gadgets.h"

static const int TheLogTagMax = 1024; // for sanity checks only


/* ILog */

ILog::ILog(): theStream(0), theTagMax(-1), theLocCount(0) {
}

ILog::~ILog() {
	while (theSymbTable.count()) delete theSymbTable.pop();
}

void ILog::stream(const String &aFileName, istream *aStream) {
	theFileName = aFileName;
	theStream = aStream;
}

// loads prefix of the current entry
// transparently skips location entries
LogEntryPrefix ILog::begEntry() {
	theCurPos = theStream->tellg();
	theCurPx.theSize = 0;
	theCurPx.theTag = 0;
	while (*this) {
		theCurPx.theSize = geti();
		theCurPx.theTag = geti();
		if (theCurPx.theTag != lgLoc)
			break;
		(*this) >> theLocTime >> theLocCount;
		endEntry();
	}
	return theCurPx;
}

void ILog::endEntry() {
	theStream->seekg(theCurPos += theCurPx.theSize);
	theCurPx.theSize = 0;
	theCurPx.theTag = 0;
}

void ILog::loadHeader() {
	const int pxPoly1 = 0x506f6c79;
	const int pxPoly2 = 0x4c6f672e;
	const int suppVer = 1;
	int h;

	if (geti() != pxPoly1 || geti() != pxPoly2 || geti() != 0) {
		cerr << here << "unknown log header format." << endl;
		exit(-1);
	}

	geti(); // current version
	if (geti(h) > suppVer) {
		cerr << here << "required version (" << h << ") too high (>" << suppVer << ")" << endl;
		exit(-1);
	}

	geti(h); // size of extra headers
	cin.ignore(h);

	// load symbol table
	loadSymbTable();
}

void ILog::loadSymbTable() {
	const LogEntryPrefix px = begEntry();
	if (px.theTag != lgSymbTable) {
		cerr << here << "expected symbol table tag, found: " << px.theTag << endl;
		exit(-1);
	}

	int h;

	geti(h); // number of symbol table entries
	//cerr << here << "loading symbol table with " << h << " entries" << endl;

	for (int i = 0; cin && i < h; ++i) {
		LogTagDescr *e = new LogTagDescr();
		e->load(*this);

		Must(e->theTag <= TheLogTagMax);
		if (e->theTag > theTagMax)
			theTagMax = e->theTag;
		else
			Must(!theSymbTable[e->theTag]); // no duplicates
		theSymbTable.put(e, e->theTag);
		//cerr << here << "symbol[" << i << "]: tag: " << e->theTag << " type: " << e->theType << endl;
	}
	endEntry();
}

int ILog::geti(int *&xs, int &count) {
	Must(geti(count) >= 0);
	xs = new int[count];
	for (int i = 0; i < count; ++i)
		geti(xs[i]);
	return count;
}

String &ILog::gets(String &s) {
	const int len = geti();
	Must(len >= 0);
	s.reset(len);
	get(s.buf(), len);
	return s;
}
