
/*
 * $Id: StatPhaseMgr.cc,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "StatPhaseMgr.h"
#include "LogComment.h"
#include "logTags.h"
#include "globals.h"

StatPhaseMgr::StatPhaseMgr(): thePhase(0), thePhaseIdx(-1) {

	thePhases
		<< new StatPhase("warm", lgStatPhaseWarm)
		<< new StatPhase("meas", lgStatPhaseMeas)
		<< new StatPhase("cool", lgStatPhaseCool);
}

StatPhaseMgr::~StatPhaseMgr() {
	thePhase = 0;
	while (thePhases.count()) delete thePhases.pop();
}


void StatPhaseMgr::goal(int aGoal) {
	Assert(!thePhase);
	theGoal = aGoal;

	thePhases[0]->goal(theGoal < 0 ? theGoal : theGoal/10);
	thePhases[1]->goal(theGoal);
	thePhases[2]->goal(theGoal);
	thePhases[2]->lock(); // last phase never exits

	nextPhase();
}

void StatPhaseMgr::forceFlush() {
	if (thePhase)
		thePhase->flush();
}

void StatPhaseMgr::noteDone(StatPhase *ph) {
	Assert(thePhase == ph);
	thePhase->flush();
	nextPhase();
}

void StatPhaseMgr::nextPhase() {
	Assert(thePhaseIdx < 2);

	thePhase = thePhases[++thePhaseIdx];
	thePhase->start(this);
}
