#ifndef CONNECTION_CLASS_H
#define CONNECTION_CLASS_H

/*
 * $Id: Connection.h,v 1.1.1.1 1999/01/27 08:40:45 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "NetAddr.h"
#include "Socket.h"
#include "Select.h"

class IOBuf {
	public:
		IOBuf();
		~IOBuf() { delete[] theBuf; }
		void reset() { theOutOff = theInOff = 0; }

		bool empty() const { return theOutOff >= theInOff; }
		bool full() const { return theInOff >= theCapacity; }

		char *space() { return theBuf + theInOff; }
		const char *content() const { return theBuf + theOutOff; }

		Size spaceSize() const { return theCapacity - theInOff; }
		Size contSize() const { return theInOff - theOutOff; }

		void appended(Size sz) { theInOff += sz; Assert(theInOff <= theCapacity); }
		void consumed(Size sz) { theOutOff += sz; if (theOutOff == theInOff) reset(); else Assert(theOutOff < theInOff); }
		void consumedAll() { reset(); }
		void pack();

	protected:
		char *theBuf;
		Size theCapacity;
		Size theOutOff;
		Size theInOff;
};

class RdBuf: public IOBuf {
};

class WrBuf: public IOBuf {
	public:
		void append(const char *buf, Size sz);
};

inline ostream &crlf(ostream &os) { return os << "\r\n"; }


class Connection {
	public:
		class HalfPipe {
			public:
				HalfPipe(Connection &aConn, IOBuf &aBuf, IODir aDir):
					theConn(aConn), theBuf(aBuf), theDir(aDir) { reset(); }

				void reset();
				void start(Select::User *u);
				void stop(Select::User *u);

			public:
				Connection &theConn;
				IOBuf &theBuf;
				IODir theDir;
				SelectReserv theReserv;
				bool isReady;
		};
		friend class HalfPipe;

	public:
		Connection();
		~Connection() { if (theSock) close(); }

		void reset();

		Socket &sock() { return theSock; }
		const NetAddr &addr() const { return theAddr; }
		bool exhausted() const { return isAtEof && !theRdBuf.contSize(); }
		bool atEof() const { return isAtEof; }
		bool bad() const { return isBad; }

		bool connect(const NetAddr &addr);
		bool accept(Socket &s, bool &fatal);
		Size read();
		Size write();
		bool close();

		void bad(bool be) { isBad = be; }

	public:
		HalfPipe theRd;
		HalfPipe theWr;

		RdBuf theRdBuf;
		WrBuf theWrBuf;

	protected:
		Socket theSock;
		NetAddr theAddr;
		//ConnMgr *theMgr;
		int theLocPort;

		Time theOpenTime;

		bool isBad;
		bool isAtEof;
};

#endif
