
/*
 * $Id: Farm.h,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef FARM_H
#define FARM_H

#include "Ring.h"

// a collection of Items that configures and recycles its members
// capacity is fixed; current recycling order is FIFO
template <class Item>
class Farm {
	public:
		Farm() {}
		virtual ~Farm() { while (!theItems.empty()) delete get(); }

		void limit(int limit) { theItems.resize(limit); }

		int capacity() const { return theItems.capacity(); }
		int level() const { return theOutLevel; }

		Item *get() { theOutLevel++; return conf(theItems.empty() ? gen() : theItems.dequeue()); }
		void put(Item *i) { theOutLevel--; theItems.full() ? delete i : theItems.enqueue(i); }

	protected:
		virtual Item *gen() { return new Item(); }
		virtual Item *conf(Item *i) { i->reset(); return i; }

	protected:
		Ring<Item*> theItems;
		int theOutLevel;    // items out there
};

#endif
