
/*
 * $Id: Histogram.h,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef HISTOGRAM_CLASS_H
#define HISTOGRAM_CLASS_H

#include "Array.h"
#include "AggrStat.h"
#include "ILog.h"
#include "OLog.h"

// general purpose histogram abstraction
// [min, max)
class Histogram {
	public:
		typedef int Val; // change to double if needed

	public:
		Histogram();
		Histogram(Val aMin, Val aMax, int binCount);
		virtual ~Histogram() {}

		void limits(Val aMin, Val aMax, int binCount);
		virtual void reset();

		void record(Val v);

		void store(OLog &log) const;
		void load(ILog &);

	protected:
		virtual const char *type() const = 0; // val2bin type
		virtual int val2Bin(Val v) const = 0; // value is already offset
		virtual Val bin2Val(int b) const = 0; // does not offset value

	protected:
		Array<int> theBins; // counters are stored here
		AggrStat theStats;
		Val theValMin;
		Val theValMax;
		int theBinMax;
};

inline OLog &operator <<(OLog &log, const Histogram &h) { h.store(log); return log; }
inline ILog &operator >>(ILog &log, Histogram &h) { h.load(log); return log; }


#endif
