
/*
 * $Id: StatPing.h,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef STAT_PING_H
#define STAT_PING_H

#include "LevelStat.h"
#include "StatInt.h"

// Accumulates "cheap" statistics (in StatPingRec) and dumps it every period

class StatPingRec {
	public:
		StatPingRec();

		void restart();
		void store(OLog &log) const;
		void load(ILog &);

		void xactBeg() { ++theXactLvl; }
		void xactEnd() { --theXactLvl; }
		void hit(Size sz) { ++theHitCnt; if (sz > 0) theHitSz += sz; }
		void miss(Size sz) { ++theMissCnt; if (sz > 0) theMissSz += sz; }

		void connOpen() { ++theOpenLvl; }
		void connEstb() { ++theEstbLvl; }
		void connClose() { --theEstbLvl; --theOpenLvl; }

		void selected() { ++theSelCnt; }
		void read(Size sz) { ++theRdCnt; if (sz > 0) theRdSz += sz; }
		void wrote(Size sz) { ++theWrCnt; if (sz > 0) theWrSz += sz; }

		void err(int /*no*/) { ++theErrCnt; }

	protected:
		LevelStat theXactLvl; // active xactions
		LevelStat theOpenLvl; // open connections
		LevelStat theEstbLvl; // estanlished connections

		int theSelCnt;  // io
		int theRdCnt;
		int theWrCnt;
		Size theRdSz;
		Size theWrSz;

		int theHitCnt;  // HR
		int theMissCnt;
		Size theHitSz;
		Size theMissSz;

		int theErrCnt;  // other

		Time theDuration; // actual int duration, computed in flush()
};

class StatPing: public StatClockInt, public StatPingRec {
	public:
		StatPing(Time aPeriod);

		virtual void flush();

	protected:
		virtual void restart();
		void report() const;
};

#endif
