
/*
 * $Id: opts.h,v 1.1.1.1 1999/01/27 08:40:45 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef OPTS_CLASSES_H
#define OPTS_CLASSES_H

#include "String.h"
#include "NetAddr.h"
#include "Opt.h"


// bool
class BoolOpt: public Opt {
	public:
		BoolOpt(OptGrp *aGrp, const char *aName, const char *aDescr, bool def = false);

		virtual bool parse(const char *name, const char *val);
		virtual void report() const;

		operator bool() const { return theVal; }

	protected:
		bool theVal;
};

// int
class IntOpt: public Opt {
	public:
		IntOpt(OptGrp *aGrp, const char *aName, const char *aDescr, int def = -1);

		virtual bool parse(const char *name, const char *val);
		virtual void report() const;

		operator int() const { return theVal; }

	protected:
		int theVal;
};

// Time
class TimeOpt: public Opt, public Time {
	public:
		TimeOpt(OptGrp *aGrp, const char *aName, const char *aDescr, Time def = Time());

		virtual bool parse(const char *name, const char *val);
		virtual void report() const;

	protected:
		Time &theVal;
};

// Size
class SizeOpt: public Opt, public Size {
	public:
		SizeOpt(OptGrp *aGrp, const char *aName, const char *aDescr, Size def = Size());

		virtual bool parse(const char *name, const char *val);
		virtual void report() const;

	protected:
		Size &theVal;
};

// String
class StrOpt: public Opt {
	public:
		StrOpt(OptGrp *aGrp, const char *aName, const char *aDescr, const char *def = 0);

		virtual bool parse(const char *name, const char *val);
		virtual void report() const;

		operator const char *() const { return theVal; }
		operator const String &() const { return theVal; }

	protected:
		String theVal;
};

// NetAddr
class NetAddrOpt: public Opt {
	public:
		NetAddrOpt(OptGrp *aGrp, const char *aName, const char *aDescr, const NetAddr &def);

		virtual bool parse(const char *name, const char *val);
		virtual void report() const;

		operator const NetAddr &() const { return theVal; }

	protected:
		NetAddr theVal;
};

// Help
class HelpOpt: public Opt {
	public:
		HelpOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual bool parse(const char *name, const char *val);
		virtual void report() const {}
		virtual bool visible() const { return false; }
};

// Two Int
class TwoIntOpt: public Opt {
	public:
		TwoIntOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual bool parse(const char *name, const char *val);
		virtual void report() const;

		bool set() const { return lo() <= hi(); }
		int hi() const { return theHiVal; }
		int lo() const { return theLoVal; }

	protected:
		int theLoVal;
		int theHiVal;
};

#endif
